#!/bin/sh
#------------------------------------------------------------------
# rhel4_gos_timeout-install.sh
#
# Author:  NetApp, Inc.
# Date:    11-JUN-2008
# Release: 3.1.80.6441
# Purpose: This script will create a udev rule on a RHEL 4 Guest OS
#          which will set the required SCSI I/O timeout (190s) for
#          connectivity to IBM N Series storage systems.  
#
# Note:    This script is required to be run by root.  
#
# Copyright (c) 2008 NetApp, Inc.
# All rights reserved.
#-------------------------------------------------------------------
IO_TIMEOUT=190
UDEV_RULE_FILE=/etc/udev/rules.d/52-ontap.rules
REINSTALL=""
RC=0

function trap_abort
{
   # Sig handler.
   echo ""
   echo "Execution aborted!"
   echo ""
   exit 1;
}

if [ -d /etc/udev/scripts ];then
   UDEV_SCRIPT=/etc/udev/scripts/scsi-disktimeout.sh
else
   UDEV_SCRIPT=/sbin/udev.scsi-disktimeout.sh
fi

# Ensure the user is running as root
USER=`/usr/bin/whoami`;

if [ "$USER" != "root" ]
then
   echo "";
   echo "This program must be run as root.";
   echo "Exiting.";
   echo "";
   exit 2;
fi;

# Verify this is RHEL4
cat /etc/redhat-release | grep "release 4" > /dev/null 2>&1;
if [[ $? -ne 0 ]]; then
   echo "";
   echo "This script is designed for Red Hat Enterprise Linux release 4.";
   echo "Please run the appropriate script for your GOS type.";
   echo "Exiting.";
   echo "";
   exit 4;
fi;

function verify_settings
{
   # Verify the timeout setting
   for device_dir in `ls -d /sys/block/sd*`
   do
      device_name=`basename ${device_dir}`
      io_tmo=`cat /sys/block/\${device_name}/device/timeout`
      echo -n "Setting I/O Timeout (${IO_TIMEOUT}s) for /dev/${device_name}"
      if [[ "${IO_TIMEOUT}" == "${io_tmo}" ]]; then
         echo " - SUCCESS!"
      else
         echo " - FAILED"
         RC=3
      fi
   done
}


function restart_udev
{
   # Restart udev to activate the new rules
   if [[ -e /sbin/udevstart ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/udevstart
      sleep 5
      verify_settings
   elif [[ -e /sbin/start_udev ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/start_udev
      sleep 5
      verify_settings
   else
      echo ""
      echo "Could not determine the correct command to restart udev."
      echo "You must reboot for the changes to take effect."
      echo ""
      RC=1
   fi;
}

function check_existing_install
{
   # Check if the settings have already been made
   if [ -e ${UDEV_RULE_FILE} ] && [ -e ${UDEV_SCRIPT} ]; then
      # Prompt to reinstall?
      echo "";
      echo "It appears the settings have already been made.";
      echo ""
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> ";
         read RESPONSE
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr -s '[:upper:]' '[:lower:]'`
      done
      if [ "$REINSTALL" = "yes" ]; then
         rm -Rf ${UDEV_SCRIPT} ${UDEV_RULE_FILE};
         # Let the script continue to let is reinstall
      else
         verify_settings;
         exit ${RC};
      fi;
   fi;
}

function create_udev_script
{
# Create the shell script that will actually modify the I/O
# Timeout setting

cat > ${UDEV_SCRIPT} <<-EOFA
	#!/bin/bash

	for device_dir in \`ls -d /sys/block/sd*\`
	do
	   device_name=\`basename \${device_dir}\`
	   echo "${IO_TIMEOUT}" > /sys/block/\${device_name}/device/timeout
	done
EOFA

# Ensure the script is executable
chmod 755 ${UDEV_SCRIPT}
}

function create_udev_rule
{
# Now, create the custom udev rule.  To avoid any upgrade
# issues if/when the udev package is upgraded, create the rule
# as its own separate file
cat > ${UDEV_RULE_FILE} <<-EOFB
	# I/O Timeout setting required for connectivity to IBM N Series storage systems
	KERNEL="sd*[a-z]", BUS="scsi", PROGRAM="${UDEV_SCRIPT}"
EOFB
}

###
### Main Program
###
trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

create_udev_script

create_udev_rule

restart_udev

exit ${RC}