#!/bin/sh
#------------------------------------------------------------------
# rhel5_gos_timeout-install.sh
#
# Author:  NetApp, Inc.
# Date:    11-JUN-2008
# Release: 3.1.80.6441
# Purpose: This script will create a udev rule on a RHEL 5 Guest OS
#          which will set the required SCSI I/O timeout (190s) for
#          connectivity to IBM N Series storage systems.  
#
# Note:    This script is required to be run by root.  
#
# Copyright (c) 2008 NetApp, Inc.
# All rights reserved.
#-------------------------------------------------------------------
IO_TIMEOUT=190
PATCH_FILE=/etc/udev/rules.d/rhel5_io_timeout.diff
UDEV_RULES=/etc/udev/rules.d/50-udev.rules
RC=0

function trap_abort
{
   # Sig handler.
   echo ""
   echo "Execution aborted!"
   echo ""
   cleanup
   exit 1;
}

# Ensure the user is running as root
USER=`/usr/bin/whoami`;

if [ "$USER" != "root" ]
then
   echo "";
   echo "This program must be run as root.";
   echo "Exiting.";
   echo "";
   exit 2;
fi;

# Verify this is RHEL5
cat /etc/redhat-release | grep "release 5" > /dev/null 2>&1;
if [[ $? -ne 0 ]]; then
   echo "";
   echo "This script is designed for Red Hat Enterprise Linux Server release 5.";
   echo "Please run the appropriate script for your GOS type.";
   echo "Exiting.";
   echo "";
   exit 4;
fi;

function cleanup
{
   rm -Rf ${PATCH_FILE};
   rm -Rf ${UDEV_RULES}.orig ${UDEV_RULES}.rej;
}

function verify_settings
{
   # Verify the timeout setting
   for device_dir in `ls -d /sys/block/sd*`
   do
      device_name=`basename ${device_dir}`
      io_tmo=`cat /sys/block/\${device_name}/device/timeout`
      echo -n "Setting I/O Timeout (${IO_TIMEOUT}s) for /dev/${device_name}"
      if [[ "${IO_TIMEOUT}" == "${io_tmo}" ]]; then
         echo " - SUCCESS!"
      else
         echo " - FAILED"
         RC=3
      fi
   done
}

function restart_udev
{
   # Restart udev to activate the new rules
   if [[ -e /sbin/start_udev ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/start_udev
      sleep 5
      verify_settings
   elif [[ -e /sbin/udevstart ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/udevstart
      sleep 5
      verify_settings
   else
      echo ""
      echo "Could not determine the correct command to restart udev."
      echo "You must reboot for the changes to take effect."
      echo ""
      RC=1
   fi;
}

function check_existing_install
{
   # Check if the settings have already been made
   grep "timeout" "${UDEV_RULES}" | grep -q ${IO_TIMEOUT};
   if [[ $? -eq 0 ]]; then
      # Prompt to reinstall?  
      echo "";
      echo "It appears this script has already been run once.";
      echo "";
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> ";
         read RESPONSE;
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr -s '[:upper:]' '[:lower:]'`;
      done;
      if [ "$REINSTALL" = "yes" ]; then
         generate_patch_diff;
         # Reverse the Patch to remove it
         patch -Rs ${UDEV_RULES} ${PATCH_FILE};
         cleanup;
         # Let the script continue to let is reinstall
      else
         verify_settings;
         exit ${RC};
      fi;
   fi;
}

function generate_patch_diff
{
# Create the shell script that will actually modify the I/O
# Timeout setting
cat >> ${PATCH_FILE} <<-EOFA
	--- 50-udev.rules	2008-02-14 14:55:57.000000000 -0500
	+++ new-50-udev.rules	2008-02-14 15:26:22.000000000 -0500
	@@ -320,7 +320,7 @@
	 # st/osst:      1 TYPE_TAPE
	 # sg: 		8 changer, [36] scanner
	 ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="0|7|14", \\
	-	RUN+="/bin/sh -c 'echo 60 > /sys\$\$DEVPATH/timeout'"
	+	RUN+="/bin/sh -c 'echo ${IO_TIMEOUT} > /sys\$\$DEVPATH/timeout'"
	 ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="1", \\
	 	RUN+="/bin/sh -c 'echo 900 > /sys\$\$DEVPATH/timeout'"
EOFA

# Ensure the script is readable
chmod 644 ${PATCH_FILE}
}


###
### Main Program
###
trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

generate_patch_diff

patch -u ${UDEV_RULES} ${PATCH_FILE} 

restart_udev

cleanup
exit ${RC}
