#!/bin/sh
#------------------------------------------------------------------
# sles9_gos_timeout-install.sh
#
# Author:  NetApp, Inc.
# Date:    11-JUN-2008
# Release: 3.1.80.6441
# Purpose: This script will create a udev rule on a SLES 9 Guest OS
#          which will set the required SCSI I/O timeout (190s) for
#          connectivity to IBM N Series storage systems.  
#
# Note:    This script is required to be run by root.  
#
# Copyright (c) 2008 NetApp, Inc.
# All rights reserved.
#-------------------------------------------------------------------
IO_TIMEOUT=190
UDEV_SCRIPT=/sbin/udev.scsi_disktimeout.sh
UDEV_RULES=/etc/udev/udev.rules
PATCH_FILE=/etc/udev/sles9_io_timeout.diff
REINSTALL=""
RC=0

#
# Sig handler.
#
function trap_abort
{
   echo ""
   echo "Execution aborted!"
   echo ""
   cleanup
   exit 1;
}

# Ensure the user is running as root
USER=`/usr/bin/whoami`;

if [ "$USER" != "root" ]
then
   echo "";
   echo "This program must be run as root.";
   echo "Exiting.";
   echo "";
   exit 2;
fi;

# Verify this is SLES9
cat /etc/SuSE-release | grep "VERSION = 9" > /dev/null 2>&1;
if [[ $? -ne 0 ]]; then
   echo "";
   echo "This script is designed for SUSE LINUX Enterprise Server 9.";
   echo "Please run the appropriate script for your GOS type.";
   echo "Exiting.";
   echo "";
   exit 4;
fi;

function cleanup
{
   rm -Rf ${PATCH_FILE};
   rm -Rf ${UDEV_RULES}.orig ${UDEV_RULES}.rej;
}

function verify_settings
{
   # Verify the timeout setting
   for device_dir in `ls -d /sys/block/sd*`;
   do
      device_name=`basename ${device_dir}`;
      io_tmo=`cat /sys/block/\${device_name}/device/timeout`;
      echo -n "Setting I/O Timeout (${IO_TIMEOUT}s) for /dev/${device_name}";
      if [[ "${IO_TIMEOUT}" == "${io_tmo}" ]]; then
         echo " - SUCCESS!";
      else
         echo " - FAILED";
         RC=3;
      fi;
   done;
}

function restart_udev
{
   # Restart udev to activate the new rules
   if [[ -e /sbin/udevstart ]]; then
      echo "";
      echo "Restarting udev... this may take a few seconds.";
      echo "";
      /sbin/udevstart;
      sleep 5;
      verify_settings;
   elif [[ -e /sbin/start_udev ]]; then
      echo "";
      echo "Restarting udev... this may take a few seconds.";
      echo "";
      /sbin/start_udev;
      sleep 5;
      verify_settings;
   else
      echo "";
      echo "Could not determine the correct command to restart udev.";
      echo "You must reboot for the changes to take effect.";
      echo "";
      RC=1;
   fi;
}


function check_existing_install
{
   # Check if the settings have already been made
   grep "${UDEV_SCRIPT}" "${UDEV_RULES}" > /dev/null 2>&1;
   if [[ $? -eq 0 ]] && [ -e ${UDEV_SCRIPT} ]; then
      # Prompt to reinstall?  
      echo "";
      echo "It appears this script has already been run once.";
      echo "";
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> ";
         read RESPONSE;
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr -s '[:upper:]' '[:lower:]'`;
      done;
      if [ "$REINSTALL" = "yes" ]; then
         rm -Rf ${UDEV_SCRIPT};
         generate_patch_diff;
         # Reverse the Patch to remove it
         patch -Rs ${UDEV_RULES} ${PATCH_FILE};
         cleanup;
         # Let the script continue to let is reinstall
      else
         verify_settings;
         exit ${RC};
      fi;
   fi;
}

function create_udev_script
{
# Create the shell script that will actually modify the I/O
# Timeout setting
cat > ${UDEV_SCRIPT} <<-EOFA
	#!/lib/klibc/bin/sh

	for device_dir in \`ls -d /sys/block/sd*\`
	do
	   device_name=\`basename \${device_dir}\`
	   echo "${IO_TIMEOUT}" > /sys/block/\${device_name}/device/timeout
	done
EOFA

# Ensure the script is executable
chmod 755 ${UDEV_SCRIPT};
}

function generate_patch_diff
{
# Create the shell script that will actually modify the I/O Timeout setting
cat >> ${PATCH_FILE} <<-EOFB
--- udev.rules  Wed Jun  4 19:23:41 2008
+++ udev.rules.new   Wed Jun  4 19:23:33 2008
@@ -8,6 +8,8 @@
 # external helpers should match /sbin/udev.*.sh
 # they should only use /lib/klibc/bin/ binaries
 #
+# I/O Timeout setting required for connectivity to IBM N Series storage systems
+BUS="scsi", PROGRAM="/sbin/udev.scsi_disktimeout.sh"
 BUS="scsi", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
 BUS="usb", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
 BUS="ide", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
EOFB

# Ensure the script is readable
chmod 644 ${PATCH_FILE};
}

###
### Main Program
###
trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

# udev requires all binaries used in the rules to be defined in the /lib/klibc/bin folder. 
cp /bin/ls  /lib/klibc/bin/ls
cp /bin/echo  /lib/klibc/bin/echo
cp /bin/basename  /lib/klibc/bin/basename

create_udev_script

generate_patch_diff

patch -u ${UDEV_RULES} ${PATCH_FILE} 

restart_udev

cleanup
exit ${RC}